// Define the dimensions of the chart
const totalWidth = 800;
const totalHeight = 800;
const padding = 40;
const chartWidth = (totalWidth - padding) / 4; // Width of each small chart
const chartHeight = (totalHeight - padding) / 4; // Height of each small chart;

// Create an SVG container for the chart
const svg = d3.select("#scatter-matrix")
    .attr("width", totalWidth)
    .attr("height", totalHeight);

// Load the Iris dataset from a CSV file
d3.csv("iris.csv").then(data => {
    // Data processing and visualization code goes here

    // Define scales and axes for the scatter plot matrix
    const xScale = d3.scaleLinear().range([padding, chartWidth - padding]);
    const yScale = d3.scaleLinear().range([chartHeight - padding, padding]);
    const xAxis = d3.axisBottom().scale(xScale);
    const yAxis = d3.axisLeft().scale(yScale);

    // Define color scale for different classes
    const colorScale = d3.scaleOrdinal(d3.schemeCategory10);

    // Create a function for drawing scatter plots

    function drawScatterPlot(xColumn, yColumn, colorColumn, row, col) {
        // Calculate the position of the small chart
        const xOffset = col * chartWidth;
        const yOffset = row * chartHeight;

        // Define the domains of x and y scales
        xScale.domain(d3.extent(data, d => +d[xColumn]));
        yScale.domain(d3.extent(data, d => +d[yColumn]));

        // Create a group for the scatter plot
        const scatter = svg.append("g")
            .attr("transform", `translate(${xOffset}, ${yOffset})`);

        // Add circles for each data point
        scatter.selectAll("circle")
            .data(data)
            .enter()
            .append("circle")
            .attr("cx", d => xScale(+d[xColumn]))
            .attr("cy", d => yScale(+d[yColumn]))
            .attr("r", 5) // Radius of circles
            .style("fill", d => colorScale(d[colorColumn]));

        // Add x and y axis
        scatter.append("g")
            .attr("class", "x-axis")
            .attr("transform", `translate(0, ${chartHeight - padding})`)
            .call(xAxis);

        scatter.append("g")
            .attr("class", "y-axis")
            .attr("transform", `translate(${padding}, 0)`)
            .call(yAxis);

        // Add labels to axes
        scatter.append("text")
            .attr("class", "x-label")
            .attr("x", chartWidth / 2)
            .attr("y", chartHeight - padding / 4)
            .text(xColumn);

        scatter.append("text")
            .attr("class", "y-label")
            .attr("transform", "rotate(-90)")
            .attr("x", -chartHeight / 2)
            .attr("y", padding / 4)
            .text(yColumn);
    }

    // Create a function for drawing histograms
    function drawHistogram(dataColumn, row, col) {
        // Calculate the position of the small chart
        const xOffset = col * chartWidth;
        const yOffset = row * chartHeight;

        // Define the domain of the x scale
        xScale.domain(d3.extent(data, d => +d[dataColumn]));

        // Create a group for the histogram
        const histogram = svg.append("g")
            .attr("transform", `translate(${xOffset}, ${yOffset})`);

        // Create a histogram generator
        const histogramGenerator = d3.histogram()
            .value(d => +d[dataColumn])
            .domain(xScale.domain())
            .thresholds(xScale.ticks(10)); // Adjust the number of bins as needed

        // Generate the histogram bins
        const bins = histogramGenerator(data);

        // Define the y scale for the histogram
        const yScaleHistogram = d3.scaleLinear()
            .domain([0, d3.max(bins, d => d.length)])
            .range([chartHeight - padding, padding]);

        // Create rectangles for the histogram bars
        histogram.selectAll("rect")
            .data(bins)
            .enter()
            .append("rect")
            .attr("x", d => xScale(d.x0))
            .attr("y", d => yScaleHistogram(d.length))
            .attr("width", d => xScale(d.x1) - xScale(d.x0) - 1) // Adjust the bar width
            .attr("height", d => chartHeight - padding - yScaleHistogram(d.length))
            .style("fill", "steelblue"); // Adjust the bar color

        // Add x axis to the histogram
        histogram.append("g")
            .attr("class", "x-axis")
            .attr("transform", `translate(0, ${chartHeight - padding})`)
            .call(xAxis);

        // Add labels to axes
        histogram.append("text")
            .attr("class", "x-label")
            .attr("x", chartWidth / 2)
            .attr("y", chartHeight - padding / 4)
            .text(dataColumn);
    }

    // Create a scatter plot matrix by iterating over columns
    const columns = ["sepal length", "sepal width", "petal length", "petal width", "class"];
    for (let i = 0; i < 4; i++) {
        for (let j = 0; j < 4; j++) {
            if (i === j) {
                // For diagonal cells, draw histograms
                const dataColumn = columns[i];
                drawHistogram(dataColumn, i, j);
            } else {
                // For non-diagonal cells, draw scatter plots
                const xColumn = columns[i];
                const yColumn = columns[j];
                drawScatterPlot(xColumn, yColumn, "class", i, j);
            }
        }
    }

    // Add a legend
    const legend = svg.append("g")
        .attr("class", "legend")
        .attr("transform", `translate(${totalWidth - padding * 2}, ${padding * 2})`);

    const legendData = data.map(d => d["class"]).filter((value, index, self) => self.indexOf(value) === index);

    const legendRectSize = 18;
    const legendSpacing = 4;

    const legendItems = legend.selectAll(".legend-item")
        .data(legendData)
        .enter()
        .append("g")
        .attr("class", "legend-item")
        .attr("transform", (d, i) => `translate(0, ${i * (legendRectSize + legendSpacing)})`);

    legendItems.append("rect")
        .attr("width", legendRectSize)
        .attr("height", legendRectSize)
        .style("fill", d => colorScale(d));

    legendItems.append("text")
        .attr("x", legendRectSize + legendSpacing)
        .attr("y", legendRectSize - legendSpacing)
        .text(d => d);
});
